﻿using DocumentFormat.OpenXml.InkML;
using PizzeriaContracts.BindingModels;
using PizzeriaContracts.ViewModels;
using PizzeriaDataModels;
using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace PizzeriaDatabaseImplement.Models
{
    public class Order
    {
        public int Id { get; private set; }
        [Required]
        public int ClientId { get; private set; }

        public virtual Client Client { get; set; } = new();

        [Required]
        public int PizzaId { get; private set; }

        public virtual Pizza Pizza { get; set; } = new();
        public int? ImplementerId { get; private set; }

        public virtual Implementer? Implementer { get; set; } = new();

        [Required]
        public int Count { get; private set; }

        [Required]
        public double Sum { get; private set; }

        [Required]
        public OrderStatus Status { get; private set; } = OrderStatus.Неизвестен;

        [Required]
        public DateTime DateCreate { get; private set; } = DateTime.Now;

        public DateTime? DateImplement { get; private set; }

        public static Order Create(PizzeriaDatabase context, OrderBindingModel model)
        {
            return new Order()
            {
                Id = model.Id,
                ClientId = model.ClientId,
                Client = context.Clients.First(x => model.ClientId == x.Id),
                PizzaId = model.PizzaId,
                Pizza = context.Pizzas.First(x => x.Id == model.PizzaId),
                ImplementerId = model.ImplementerId,
                Implementer = model.ImplementerId.HasValue ? context.Implementers.First(x => x.Id == model.ImplementerId) : null,
                Count = model.Count,
                Sum = model.Sum,
                Status = model.Status,
                DateCreate = model.DateCreate,
                DateImplement = model.DateImplement,
            };
        }

        public void Update(PizzeriaDatabase context, OrderBindingModel? model)
        {
            if (model == null)
            {
                return;
            }
            Status = model.Status;
            DateImplement = model.DateImplement;
            ImplementerId = model.ImplementerId;
            Implementer = model.ImplementerId.HasValue ? context.Implementers.First(x => x.Id == model.ImplementerId) : null;
        }

        public OrderViewModel GetViewModel => new()
        {
            Id = Id,
            ClientId = ClientId,
            ClientFIO = Client.ClientFIO,
            PizzaId = PizzaId,
            PizzaName = Pizza.PizzaName,
            ImplementerId = ImplementerId,
            ImplementerFIO = Implementer != null ? Implementer.ImplementerFIO : null,
            Count = Count,
            Sum = Sum,
            Status = Status,
            DateCreate = DateCreate,
            DateImplement = DateImplement,
        };
    }
}
