﻿using PizzeriaContracts.BindingModels;
using PizzeriaContracts.ViewModels;
using System.Xml.Linq;
using PizzeriaDataModels;

namespace PizzeriaFileImplement.Models
{
    public class Order : IOrderModel
    {
        public int Id { get; private set; }
        public int ClientId { get; private set; }
        public int PizzaId { get; private set; }
        public int? ImplementerId { get; private set; }
        public int Count { get; private set; }
        public double Sum { get; private set; }
        public OrderStatus Status { get; private set; } = OrderStatus.Неизвестен;
        public DateTime DateCreate { get; private set; } = DateTime.Now;
        public DateTime? DateImplement { get; private set; }
        public static Order? Create(OrderBindingModel? model)
        {
            if (model == null)
            {
                return null;
            }
            return new Order()
            {
                Id = model.Id,
                ClientId = model.ClientId,
                PizzaId = model.PizzaId,
                ImplementerId = model.ImplementerId,
                Count = model.Count,
                Sum = model.Sum,
                Status = model.Status,
                DateCreate = model.DateCreate,
                DateImplement = model.DateImplement,
            };
        }
        public static Order? Create(XElement element)
        {
            if (element == null)
            {
                return null;
            }
            string dateImplement = element.Element("DateImplement")!.Value;
            return new Order()
            {
                Id = Convert.ToInt32(element.Attribute("Id")!.Value),
                ClientId= Convert.ToInt32(element.Attribute("ClientId")!.Value),
                PizzaId = Convert.ToInt32(element.Element("PizzaId")!.Value),
                ImplementerId = Convert.ToInt32(element.Element("ImplementerId")!.Value),
                Count = Convert.ToInt32(element.Element("Count")!.Value),
                Sum = Convert.ToDouble(element.Element("Sum")!.Value),
                Status = (OrderStatus)(Enum.Parse(typeof(OrderStatus), element.Element("Status")!.Value)),
                DateCreate = Convert.ToDateTime(element.Element("DateCreate")!.Value),
                DateImplement = (dateImplement == "" || dateImplement is null) ? Convert.ToDateTime(null) : Convert.ToDateTime(dateImplement)
            };

        }
        public void Update(OrderBindingModel? model)
        {
            if (model == null)
            {
                return;
            }
            ImplementerId = model.ImplementerId;
            Status = model.Status;
            if (model.Status == OrderStatus.Выдан) DateImplement = model.DateImplement;
        }
        public OrderViewModel GetViewModel => new()
        {
            Id = Id,
            ClientId = ClientId,
            PizzaId = PizzaId,
            ImplementerId = ImplementerId,
            Count = Count,
            Sum = Sum,
            Status = Status,
            DateCreate = DateCreate,
            DateImplement = DateImplement,
        };

        public XElement GetXElement => new("Order",
           new XAttribute("Id", Id),
           new XElement("ClientId",ClientId.ToString()),
           new XElement("PizzaId", PizzaId.ToString()),
           new XElement("ImplementerId", ImplementerId),
           new XElement("Count", Count.ToString()),
           new XElement("Sum", Sum.ToString()),
           new XElement("Status", Status.ToString()),
           new XElement("DateCreate", DateCreate.ToString()),
           new XElement("DateImplement", DateImplement.ToString()));

    }
}
