﻿using PizzeriaContracts.BindingModels;
using PizzeriaContracts.BusinessLogicsContracts;
using Microsoft.Extensions.Logging;

namespace Pizzeria
{
	public partial class FormClients : Form
	{
		private readonly ILogger _logger;
		private readonly IClientLogic _logic;

		public FormClients(ILogger<FormClients> logger, IClientLogic logic)
		{
			InitializeComponent();
			_logger = logger;
			_logic = logic;
		}

		private void FormClients_Load(object sender, EventArgs e)
		{
			LoadData();
		}

		private void LoadData()
		{
			try
			{
				var list = _logic.ReadList(null);
				if (list != null)
				{
					dataGridViewClients.DataSource = list;
					dataGridViewClients.Columns["Id"].Visible = false;
					dataGridViewClients.Columns["ClientFIO"].AutoSizeMode = DataGridViewAutoSizeColumnMode.Fill;
				}
				_logger.LogInformation("Загрузка клиентов");
			}
			catch (Exception ex)
			{
				_logger.LogError(ex, "Ошибка загрузки клиентов");
				MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
			}
		}

		private void ButtonDel_Click(object sender, EventArgs e)
		{
			if (dataGridViewClients.SelectedRows.Count == 1)
			{
				if (MessageBox.Show("Удалить запись?", "Вопрос", MessageBoxButtons.YesNo, MessageBoxIcon.Question) ==
					DialogResult.Yes)
				{
					int id = Convert.ToInt32(dataGridViewClients.SelectedRows[0].Cells["Id"].Value);
					_logger.LogInformation("Удаление клиента");
					try
					{
						if (!_logic.Delete(new ClientBindingModel { Id = id }))
						{
							throw new Exception("Ошибка при удалении. Дополнительная информация в логах.");
						}
						LoadData();
					}
					catch (Exception ex)
					{
						_logger.LogError(ex, "Ошибка удаления клиента");
						MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
					}
				}
			}
		}

		private void ButtonRef_Click(object sender, EventArgs e)
		{
			LoadData();
		}
	}
}
